import { Player, world, system, CustomCommandParamType, CustomCommandStatus, CommandPermissionLevel } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";

const colorList = ["white", "light gray", "gray", "black", "brown", "red", "orange", "yellow", "lime", "green", "cyan", "aqua", "blue", "purple", "magenta", "pink"]

world.afterEvents.worldLoad.subscribe(() => world.sendMessage(`§eThanks for installing §6Glowing Addon V5 by TrayePlays\n§eIf you need any assistance for the mod join the discord: §9discord.gg/GHzNqpZ4Bu`))

system.beforeEvents.startup.subscribe(({ customCommandRegistry }) => {
    const trayeGlow = {
        name: "traye:glow",
        description: "Sets glow for others",
        permissionLevel: CommandPermissionLevel.GameDirectors,
        mandatoryParameters: [
            { type: CustomCommandParamType.PlayerSelector, name: "Player" },
            { type: CustomCommandParamType.Enum, name: "traye:color_list" }
        ]
    };
    const trayeGlowToggle = {
        name: "traye:glowsettings",
        description: "Sets glow on or off for others",
        permissionLevel: CommandPermissionLevel.GameDirectors,
        mandatoryParameters: [
            { type: CustomCommandParamType.PlayerSelector, name: "Player" },
            { type: CustomCommandParamType.Enum, name: "traye:setting"},
            { type: CustomCommandParamType.Boolean, name: "On / Off" }
        ]
    };
    const setting = ["behind_walls", "toggle_glow"]
    customCommandRegistry.registerEnum("traye:setting", setting)
    // customCommandRegistry.registerEnum("better:equipmentSlot", ["Head", "Chest", "Legs", "Feet", "Mainhand", "Offhand", "None"]);
    customCommandRegistry.registerEnum("traye:color_list", colorList);
    customCommandRegistry.registerCommand(trayeGlowToggle, glowSettings)
    customCommandRegistry.registerCommand(trayeGlow, setGlow);
});

function glowSettings(origin, players, setting, toggle) {
    system.run(() => {
        for (const player of players) {
            const toggleOption = toggle == true ? 1 : 0
            if (setting == "behind_walls") {
                player.setProperty("traye:behind_wall", toggleOption)
            }
            else if (setting == "toggle_glow") {
                player.setProperty("traye:glow_on", toggleOption)
            }
            else {
                origin.sourceEntity.sendMessage(`Choose one of the options: behind_walls or toggle_glow`)
                return {
                    status: CustomCommandStatus.Failure
                }
            }
        }
        return {
            status: CustomCommandStatus.Success
        }
    })
}

function setGlow(origin, players, color) {
    system.run(() => {
        for (const player of players) {
            const index = colorList.findIndex(a => a == color)
            if (index == undefined) {
                origin.sourceEntity.sendMessage(`Choose one of the options: behind_walls or toggle_glow`)
                return {
                    status: CustomCommandStatus.Failure
                }
            }
            else {
                player.setProperty("traye:glow", index)
                return {
                    status: CustomCommandStatus.Success
                }
            }
        }
    })
}

system.afterEvents.scriptEventReceive.subscribe(({id, sourceEntity, message}) => {
    if (id == "traye:glow_color") {
        if (colorList.includes(message.toLowerCase())) {
            const index = colorList.findIndex(a => a == message.toLowerCase())
            try {
                sourceEntity.setProperty("traye:glow", index)
            }
            catch (er) {
                world.sendMessage(`§4ERROR: §c${er}`)
            }
        }
        else {
            world.sendMessage(`§4ERROR: §cexpected a color\n§7Ex: /scriptevent traye:glow_color §6light gray`)
        }
    }
    else if (id == "traye:glow_on") {
        if (message.toLowerCase() == "true") {
            try {
                sourceEntity.setProperty("traye:glow_on", 1)
            }
            catch (er) {
                world.sendMessage(`§4ERROR: §c${er}`)
            }
        }
        else if (message.toLowerCase() == "false") {
            try {
                sourceEntity.setProperty("traye:glow_on", 0)
            }
            catch (er) {
                world.sendMessage(`§4ERROR: §c${er}`)
            }
        }
        else {
            world.sendMessage(`§4ERROR: §cexpected 'true' or 'false'\n§7Ex: /scriptevent traye:glow_on §6true`)
        }
    }
    else if (id == "traye:behind_wall") {
        if (message.toLowerCase() == "true") {
            try {
                sourceEntity.setProperty("traye:behind_wall", 1)
            }
            catch (er) {
                world.sendMessage(`§4ERROR: §c${er}`)
            }
        }
        else if (message.toLowerCase() == "false") {
            try {
                sourceEntity.setProperty("traye:behind_wall", 0)
            }
            catch (er) {
                world.sendMessage(`§4ERROR: §c${er}`)
            }
        }
        else {
            world.sendMessage(`§4ERROR: §cexpected 'true' or 'false'\n§7Ex: /scriptevent traye:behind_wall §6true`)
        }
    }
    else if (id.startsWith("traye:")) {
        world.sendMessage(`§4ERROR: §cThis script event doesn't exist in the Glowing BP`)
    }
})

/** @param {Player} player  */
async function openMenu(player) {
    const form = new ActionFormData()
    form.title(`Settings`)
    form.button("Change Color", "textures/ui/color_picker")
    form.button("Toggle Behind Walls", player.getProperty("traye:behind_wall") == 1 ? "textures/ui/friend_glyph_desaturated" : "textures/ui/Friend1")
    form.button("Toggle Glow", player.getProperty("traye:glow_on") == 1 ? "textures/ui/friend_glyph_desaturated" : "textures/ui/Friend1")
    form.button("Extra Info", "textures/ui/infobulb")
    const {canceled, selection} = await form.show(player)
    if (canceled) return
    if (selection == 0) {
        openChangeColorMenu(player)
    }
    if (selection == 1) {
        if (player.getProperty("traye:behind_wall") == 1) {
            player.sendMessage("§cBehind Walls Off")
            player.setProperty("traye:behind_wall", 0)
            return
        }
        if (player.getProperty("traye:behind_wall") == 0) {
            player.sendMessage("§aBehind Walls On")
            player.setProperty("traye:behind_wall", 1)
            return
        }
    }
    if (selection == 2) {
        if (player.getProperty("traye:glow_on") == 1) {
            player.sendMessage("§cGlow Off")
            player.setProperty("traye:glow_on", 0)
            return
        }
        if (player.getProperty("traye:glow_on") == 0) {
            player.sendMessage("§aGlow On")
            player.setProperty("traye:glow_on", 1)
            return
        }
    }
    if (selection == 3) openExtraInfoMenu(player)
}

async function openExtraInfoMenu(player) {
    const form = new ActionFormData()
    form.title(`Extra Info`)
    form.body(`§dCommand List:\n§e-/traye:glow §7<player> <color>\n§e-/traye:glowsettings §7<player> <option> <boolean>\n§l§8--------------------------------------\n§r§aScript Events (Old Method)\n§e-/scriptevent traye:glow_on §7<boolean>§e\n-/scriptevent traye:behind_wall §7<boolean>§e\n-/scriptevent traye:glow_color §7<color>§e\n§l§8--------------------------------------\n§r§fYou can do '§e/execute as §7<player>§f' to run these commands in command blocks or as other players\n§4§lONLY WORKS ON PLAYERS\n§l§8--------------------------------------\n§r§bBoolean = True or False`)
    form.button("Okay")
    const { canceled } = await form.show(player)
    if (canceled) return
    openMenu(player)
}

async function openChangeColorMenu(player) {
    const form = new ActionFormData()
    form.title(`Glow Color`)
    const colorList = ["White", "Light Gray", "Gray", "Black", "Brown", "Red", "Orange", "Yellow", "Lime", "Green", "Cyan", "Aqua", "Blue", "Purple", "Magenta", "Pink"]
    form.button("White", `textures/blocks/concrete_white`)
    form.button("Light Gray", `textures/blocks/concrete_silver`)
    form.button("Gray", `textures/blocks/concrete_gray`)
    form.button("Black", `textures/blocks/concrete_black`)
    form.button("Brown", `textures/blocks/concrete_brown`)
    form.button("Red", `textures/blocks/concrete_red`)
    form.button("Orange", `textures/blocks/concrete_orange`)
    form.button("Yellow", `textures/blocks/concrete_yellow`)
    form.button("Lime", `textures/blocks/concrete_lime`)
    form.button("Green", `textures/blocks/concrete_green`)
    form.button("Cyan", `textures/blocks/concrete_cyan`)
    form.button("Aqua", `textures/blocks/concrete_light_blue`)
    form.button("Blue", `textures/blocks/concrete_blue`)
    form.button("Purple", `textures/blocks/concrete_purple`)
    form.button("Magenta", `textures/blocks/concrete_magenta`)
    form.button("Pink", `textures/blocks/concrete_pink`)
    const {canceled, selection} = await form.show(player)
    if (canceled) return
    player.sendMessage(`§eChanged §6${player.name}'s§e color to ${colorList[selection]}`)
    player.setProperty("traye:glow", selection)
}

world.afterEvents.itemUse.subscribe(({itemStack, source: player}) => {
    if (itemStack?.typeId == "traye:menu") openMenu(player)
})